%\BeginDoc
%
%\Name: cycles
%
%\Description: 
%     Exhaustive computation of cycles of the matroid
%     defined by the null space of a stoichiometric matrix S.
%
%\Author
%     Daniel A. Beard
%     Feng Yang
%     Biotech. & Bioengineering Center 
%     Medical College of Wisconsin, Milwaukee, Wisconsin
%
%\Remarks
%     1. Version1.1 was created at July 2, 2004.
%     2. Version2.1 was availabe at July 12, 2004.
%
%\References:
%  1. Beard DA, Liang SD, Qian H: Energy balance for analysis of complex 
%     metabolic networks. Biophys J 2002, 83:79-86.
%  2. Qian H, Beard DA, Liang SD: Stoichiometric network theory for 
%     nonequilibrium bio-chemical systems. Eur J Biochem 2003, 270:415-421.
%  3. Beard DA, Babson E, Curtis E, Qian H: Thermodynamic constraints for 
%     biochemical networks. J Theor Biol 2004, 228:327-333.
%
%\EndDoc
%
%-------------------------------------------------------------------------
%
%\Arguments
%   S          Stoichiometric matrix; containts the stoichiometric numbers
%              for reactions in a reaction network. (integer or double)
%
%\Program outputs:
%   N          Set of vectors in the null space of S that
%              represent minimal cycle basis.
%
 
    function [N] = cycles(S);
%    
%  %--------------------------------------%
%  | Compute a basis for null space of S. |
%  %--------------------------------------%
%   
    N = null(S,'r');
    [m,n] = size(N);

    i = 1;  while i < n
      j = i+1;  while j <= n
%    
%     %----------------------------------------------------------%
%     | Compute new null space vectors from linear combinations  |
%     | of i^th and j^th columns of N:                           |
%     %----------------------------------------------------------%
%          
      Nnew = linear_combos( N(:,i), N(:,j) );
%    
%     %--------------------------------------------------------------%
%     | Test new null space vectors (Nnew) to see if they form       |
%     | cycles that are minimal against each other and vectors in N. | 
%     | The set of vectors N is appended with the new orthogonal     |
%     | entries:                                                     |
%     %--------------------------------------------------------------%
% 
      if size(Nnew,2) ~= 0            
          N = test_new_vectors(N,Nnew);  
      end    
      
      n = length(N(1,:));        
      j = j+1;     end 
    i = i+1;    end
%    
%  %-------------------------%
%  | End of cycles function  |
%  %-------------------------%
%  
%
%\BeginDoc
%
%\Name: linear_combos 
%
%\Description: 
%   This code make a linear combination of two vectors. 
%
%\Arguments
%   v1,v2      Vectors with dimention n. 
%
%\Program outputs:
%   Nnew       Contains linear combinations of v1 and v2.  
%              
%\EndDoc
%
%-------------------------------------------------------------------------    
%        
    function [Nnew] = linear_combos(v1,v2);
%    
%   %-----------------%
%   | Initialization. | 
%   %-----------------%
%   
    Nnew = [];    
%    
%   %--------------------------------------------------------------------%
%   | Make sure v1 and v2 are not  proper subsets of v1+v2 or v1-v2.     |
%   %--------------------------------------------------------------------%
%      
    if intial_test((v1+v2), v1, v2) == true 
       Nnew = [Nnew (v1+v2)];
    end
    
    if intial_test((v1-v2), v1, v2) == true 
       Nnew = [Nnew (v1-v2)];
    end    
%    
%   %--------------------------------------------------------------------%
%   | Construct new linear combination so that its i-th entry is zero;   |
%   | also gaurantee v1 and v2 are not proper subsets of this vector.    |
%   %--------------------------------------------------------------------%
%     
    n = length(v1);
    for i = 1:n
        v1i = v1(i);
        v2i = v2(i);
        if (v1i ~= 0) & (v2i ~= 0) & ( (abs(v1i)~=1)|(abs(v2i)~=1) )
          addone = (v2i.*v1 - v1i.*v2);         
          if intial_test(addone, v1, v2) == true 
             Nnew = [Nnew addone];
          end          
        end            
    end
    
%   Safeguard w.r.t. numerical error.     
    epsilon = 1e-10;
    Nnew = Nnew.*( abs(Nnew) > epsilon );
%    
%  %------------------------------------%
%  | End of linear_combos function      |
%  %------------------------------------%
%

%\BeginDoc
%
%\Name: intial_test 
%
%\Description: 
%   This code make an intial test to ensure that v1 and v2 are not 
%   proper subsets of vnew.  
%
%\Arguments
%   vnew       Vectors to be tested again v1 and v2.
%   v1,v2      Vectors with dimention n. 
%
%\Program outputs:
%   TF         Boolean value. 
%              
%\EndDoc
%
   function [TF] = intial_test(vnew, v1, v2)
       
    N1_agree = sum( abs(sign(vnew)) .* abs(sign(v1)) ) ;   
    N2_agree = sum( abs(sign(vnew)) .* abs(sign(v2)) ) ;        
    if N1_agree ~= sum( v1 ~= 0 ) & N2_agree ~= sum( v2 ~= 0 )
       TF = true ;
    else
       TF = false;
    end
%    
%  %------------------------------------%
%  | End of intial_test function        |
%  %------------------------------------%
%    
%  
%
%\BeginDoc
%
%\Name: test_new_vectors 
%
%\Description: 
%    Function appends the set of column vectors
%    N with members in the set Nnew that are minimal cycles.
%
%\Arguments
%   N, Nnew     Nnew to be tested with respect to N. 
%
%\Program outputs:
%   N           Filtered column vectors of Nnew were appended into N.        
%              
%\EndDoc
%
%-------------------------------------------------------------------------    
%        
    function [N] = test_new_vectors(N,Nnew)
%    
%  %------------------------------------%
%  | Change to sign vector.             |
%  %------------------------------------%
%        
    Ns = abs(sign(N));
    Nsnew = abs(sign(Nnew));
%    
%  %-------------------------------------------------%
%  | Each cycle should has minimum signed supports.  |
%  %-------------------------------------------------%
%    
    for j = 1:size(Nsnew,2) 
        t = Ns'*Nsnew(:,j) - sum(Ns~=0,1)';                             
        if sum(t~=0) == length(t) & sum( Nsnew(:,j)~=0) ~= 0
            N = [N Nnew(:,j)];
            Ns = [Ns Nsnew(:,j)];
        end 
    end
%    
%  %---------------------------------------%
%  | End of test_new_vectors function      |
%  %---------------------------------------%
% 


